/* $Id: dflow.c,v 1.2 2002/07/24 10:07:47 aa5779 Exp $ */
/*
	Copyright (C) 2001, 2002 Artem V. Andreev (artem@AA5779.spb.edu)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include "interp.h"
#include "shared.h"
#include "flow.h"
#include "bstream.h"
#define DFLOW_IMP
#include "dflow.h"
#include "util.h"

flow_class dup_flow_class = {"script", df_open, df_close, df_getc, df_ungetc, 
							   df_putc, df_puts,
							   df_getpos, df_setpos, df_rewind,
							   df_flowctl
							 };

typedef struct dup_flow_data
{
	flow_class *cls;
	flow_handle_t data;
} dup_flow_data;

flow_handle_t df_open(char *name, char *options, int writeable)
{
	stream *base_stream = find_exported("stream", name);
	dup_flow_data *dfd = ALLOC(dup_flow_data);
	if(!base_stream)
	{
		FREE(dfd);
		recover_error("can't find exported stream '%s'", name);
	}
	if(!*base_stream->class->ioctl(NULL, base_stream, "providesbasic"))
		recover_error("stream '%s' is not derived from basic", name);
	dfd->cls  = ((basic_stream *)base_stream->data)->fc;
	dfd->data  = ((basic_stream *)base_stream->data)->fh;
	return dfd;
}

void df_close(flow_handle_t fh)
{
	dup_flow_data *dfd = fh;
	FREE(dfd);
}

int df_getc(flow_handle_t fh)
{
	dup_flow_data *dfd = fh;
	return (dfd->cls->getc)(dfd->data);
}

int df_ungetc(flow_handle_t fh, int ch)
{
	dup_flow_data *dfd = fh;
	return (dfd->cls->ungetc)(dfd->data, ch);
}

int df_putc(flow_handle_t fh, int ch)
{
	dup_flow_data *dfd = fh;
	return (dfd->cls->putc)(dfd->data, ch);
}

int df_puts(flow_handle_t fh, char *str)
{
	dup_flow_data *dfd = fh;
	return (dfd->cls->puts)(dfd->data, str);
}
 
flow_pos_t df_getpos(flow_handle_t fh)
{
	dup_flow_data *dfd = fh;
	return dfd->cls->getpos(dfd->data);
}

int df_setpos(flow_handle_t fh, flow_pos_t fp)
{
	dup_flow_data *dfd = fh;
	return dfd->cls->setpos(dfd->data, fp);
}

int df_rewind(flow_handle_t fh)
{
	dup_flow_data *dfd = fh;
	return dfd->cls->rewind(dfd->data);
}

char *df_flowctl(flow_handle_t fh, char *ctl)
{
	dup_flow_data *dfd = fh;
	return dfd->cls->flowctl(dfd->data, ctl);
}
